@@ -1,16 +1,15 @@
 """
 Acquisition functions for Bayesian optimization.
 """
-from typing import Tuple
 
 import numpy as np
-from scipy.stats import norm
 from scipy.special import ndtr
+from scipy.stats import norm
 from sklearn.exceptions import NotFittedError
 
-from modAL.utils.selection import multi_argmax
-from modAL.utils.data import modALinput
 from modAL.models.base import BaseLearner
+from modAL.utils.data import modALinput
+from modAL.utils.selection import multi_argmax
 
 
 def PI(mean, std, max_val, tradeoff):
@@ -104,7 +103,7 @@ Query strategies using acquisition functions
 
 
 def max_PI(optimizer: BaseLearner, X: modALinput, tradeoff: float = 0,
-           n_instances: int = 1) -> Tuple[np.ndarray, modALinput]:
+           n_instances: int = 1) -> np.ndarray:
     """
     Maximum PI query strategy. Selects the instance with highest probability of improvement.
 
@@ -115,16 +114,16 @@ def max_PI(optimizer: BaseLearner, X: modALinput, tradeoff: float = 0,
         n_instances: Number of samples to be queried.
 
     Returns:
-        The indices of the instances from X chosen to be labelled; the instances from X chosen to be labelled.
+        The indices of the instances from X chosen to be labelled.
+        The pi metric of the chosen instances.
+
     """
     pi = optimizer_PI(optimizer, X, tradeoff=tradeoff)
-    query_idx = multi_argmax(pi, n_instances=n_instances)
-
-    return query_idx, X[query_idx]
+    return multi_argmax(pi, n_instances=n_instances)
 
 
 def max_EI(optimizer: BaseLearner, X: modALinput, tradeoff: float = 0,
-           n_instances: int = 1) -> Tuple[np.ndarray, modALinput]:
+           n_instances: int = 1) -> np.ndarray:
     """
     Maximum EI query strategy. Selects the instance with highest expected improvement.
 
@@ -135,16 +134,16 @@ def max_EI(optimizer: BaseLearner, X: modALinput, tradeoff: float = 0,
         n_instances: Number of samples to be queried.
 
     Returns:
-        The indices of the instances from X chosen to be labelled; the instances from X chosen to be labelled.
+        The indices of the instances from X chosen to be labelled. 
+        The ei metric of the chosen instances.
+
     """
     ei = optimizer_EI(optimizer, X, tradeoff=tradeoff)
-    query_idx = multi_argmax(ei, n_instances=n_instances)
-
-    return query_idx, X[query_idx]
+    return multi_argmax(ei, n_instances=n_instances)
 
 
 def max_UCB(optimizer: BaseLearner, X: modALinput, beta: float = 1,
-            n_instances: int = 1) -> Tuple[np.ndarray, modALinput]:
+            n_instances: int = 1) -> np.ndarray:
     """
     Maximum UCB query strategy. Selects the instance with highest upper confidence bound.
 
@@ -155,9 +154,9 @@ def max_UCB(optimizer: BaseLearner, X: modALinput, beta: float = 1,
         n_instances: Number of samples to be queried.
 
     Returns:
-        The indices of the instances from X chosen to be labelled; the instances from X chosen to be labelled.
+        The indices of the instances from X chosen to be labelled. 
+        The ucb metric of the chosen instances.
+
     """
     ucb = optimizer_UCB(optimizer, X, beta=beta)
-    query_idx = multi_argmax(ucb, n_instances=n_instances)
-
-    return query_idx, X[query_idx]
+    return multi_argmax(ucb, n_instances=n_instances)
