@@ -6,11 +6,12 @@ from typing import Callable, Optional, Tuple, Union
 
 import numpy as np
 import scipy.sparse as sp
-from sklearn.metrics.pairwise import pairwise_distances, pairwise_distances_argmin_min
+from sklearn.metrics.pairwise import (pairwise_distances,
+                                      pairwise_distances_argmin_min)
 
-from modAL.utils.data import data_vstack, modALinput
 from modAL.models.base import BaseCommittee, BaseLearner
 from modAL.uncertainty import classifier_uncertainty
+from modAL.utils.data import data_shape, data_vstack, modALinput
 
 
 def select_cold_start_instance(X: modALinput,
@@ -114,7 +115,7 @@ def select_instance(
     unlabeled_indices = [i for i in range(n_pool) if mask[i]]
     best_instance_index = unlabeled_indices[best_instance_index_in_unlabeled]
     mask[best_instance_index] = 0
-    return best_instance_index, np.expand_dims(X_pool[best_instance_index], axis=0), mask
+    return best_instance_index, X_pool[[best_instance_index]], mask
 
 
 def ranked_batch(classifier: Union[BaseLearner, BaseCommittee],
@@ -139,21 +140,30 @@ def ranked_batch(classifier: Union[BaseLearner, BaseCommittee],
 
     Returns:
         The indices of the top n_instances ranked unlabelled samples.
+        The uncertainty scores of the chosen instances. 
+
     """
     # Make a local copy of our classifier's training data.
     # Define our record container and record the best cold start instance in the case of cold start.
+
+    # transform unlabeled data if needed
+    if classifier.on_transformed:
+        unlabeled = classifier.transform_without_estimating(unlabeled)
+
     if classifier.X_training is None:
         best_coldstart_instance_index, labeled = select_cold_start_instance(X=unlabeled, metric=metric, n_jobs=n_jobs)
         instance_index_ranking = [best_coldstart_instance_index]
-    elif classifier.X_training.shape[0] > 0:
-        labeled = classifier.X_training[:]
+    elif data_shape(classifier.X_training)[0] > 0:
+        labeled = classifier.transform_without_estimating(
+            classifier.X_training
+        ) if classifier.on_transformed else classifier.X_training[:]
         instance_index_ranking = []
     
     # The maximum number of records to sample.
     ceiling = np.minimum(unlabeled.shape[0], n_instances) - len(instance_index_ranking)
 
     # mask for unlabeled initialized as transparent
-    mask = np.ones(unlabeled.shape[0], np.bool)
+    mask = np.ones(unlabeled.shape[0], bool)
 
     for _ in range(ceiling):
 
@@ -171,7 +181,7 @@ def ranked_batch(classifier: Union[BaseLearner, BaseCommittee],
         instance_index_ranking.append(instance_index)
 
     # Return numpy array, not a list.
-    return np.array(instance_index_ranking)
+    return np.array(instance_index_ranking), uncertainty_scores[np.array(instance_index_ranking)]
 
 
 def uncertainty_batch_sampling(classifier: Union[BaseLearner, BaseCommittee],
@@ -180,7 +190,7 @@ def uncertainty_batch_sampling(classifier: Union[BaseLearner, BaseCommittee],
                                metric: Union[str, Callable] = 'euclidean',
                                n_jobs: Optional[int] = None,
                                **uncertainty_measure_kwargs
-                               ) -> Tuple[np.ndarray, Union[np.ndarray, sp.csr_matrix]]:
+                               ) -> np.ndarray:
     """
     Batch sampling query strategy. Selects the least sure instances for labelling.
 
@@ -203,9 +213,12 @@ def uncertainty_batch_sampling(classifier: Union[BaseLearner, BaseCommittee],
         **uncertainty_measure_kwargs: Keyword arguments to be passed for the :meth:`predict_proba` of the classifier.
 
     Returns:
-        Indices of the instances from `X` chosen to be labelled; records from `X` chosen to be labelled.
+        Indices of the instances from `X` chosen to be labelled
+        Records from `X` chosen to be labelled.
+        The uncertainty scores of the chosen instances. 
+
     """
     uncertainty = classifier_uncertainty(classifier, X, **uncertainty_measure_kwargs)
-    query_indices = ranked_batch(classifier, unlabeled=X, uncertainty_scores=uncertainty,
+    return ranked_batch(classifier, unlabeled=X, uncertainty_scores=uncertainty,
                                  n_instances=n_instances, metric=metric, n_jobs=n_jobs)
-    return query_indices, X[query_indices]
+
