@@ -2,16 +2,15 @@
 Disagreement measures and disagreement based query strategies for the Committee model.
 """
 from collections import Counter
-from typing import Tuple
 
 import numpy as np
 from scipy.stats import entropy
-from sklearn.exceptions import NotFittedError
 from sklearn.base import BaseEstimator
+from sklearn.exceptions import NotFittedError
 
+from modAL.models.base import BaseCommittee
 from modAL.utils.data import modALinput
 from modAL.utils.selection import multi_argmax, shuffled_argmax
-from modAL.models.base import BaseCommittee
 
 
 def vote_entropy(committee: BaseCommittee, X: modALinput, **predict_proba_kwargs) -> np.ndarray:
@@ -35,7 +34,6 @@ def vote_entropy(committee: BaseCommittee, X: modALinput, **predict_proba_kwargs
         return np.zeros(shape=(X.shape[0],))
 
     p_vote = np.zeros(shape=(X.shape[0], len(committee.classes_)))
-    entr = np.zeros(shape=(X.shape[0],))
 
     for vote_idx, vote in enumerate(votes):
         vote_counter = Counter(vote)
@@ -43,8 +41,7 @@ def vote_entropy(committee: BaseCommittee, X: modALinput, **predict_proba_kwargs
         for class_idx, class_label in enumerate(committee.classes_):
             p_vote[vote_idx, class_idx] = vote_counter[class_label]/n_learners
 
-        entr[vote_idx] = entropy(p_vote[vote_idx])
-
+    entr = entropy(p_vote, axis=1)
     return entr
 
 
@@ -118,8 +115,9 @@ def vote_entropy_sampling(committee: BaseCommittee, X: modALinput,
             measure function.
 
     Returns:
-        The indices of the instances from X chosen to be labelled;
-         the instances from X chosen to be labelled.
+        The indices of the instances from X chosen to be labelled.
+        The disagrerment metric of the chosen instances. 
+
     """
     disagreement = vote_entropy(committee, X, **disagreement_measure_kwargs)
 
@@ -145,8 +143,9 @@ def consensus_entropy_sampling(committee: BaseCommittee, X: modALinput,
             measure function.
 
     Returns:
-        The indices of the instances from X chosen to be labelled;
-        the instances from X chosen to be labelled.
+        The indices of the instances from X chosen to be labelled.
+        The disagrerment metric of the chosen instances. 
+
     """
     disagreement = consensus_entropy(committee, X, **disagreement_measure_kwargs)
 
@@ -172,8 +171,9 @@ def max_disagreement_sampling(committee: BaseCommittee, X: modALinput,
          measure function.
 
     Returns:
-        The indices of the instances from X chosen to be labelled;
-        the instances from X chosen to be labelled.
+        The indices of the instances from X chosen to be labelled.
+        The disagrerment metric of the chosen instances. 
+
     """
     disagreement = KL_max_disagreement(committee, X, **disagreement_measure_kwargs)
 
@@ -198,8 +198,9 @@ def max_std_sampling(regressor: BaseEstimator, X: modALinput,
         **predict_kwargs: Keyword arguments to be passed to :meth:`predict` of the CommiteeRegressor.
 
     Returns:
-        The indices of the instances from X chosen to be labelled;
-        the instances from X chosen to be labelled.
+        The indices of the instances from X chosen to be labelled.
+        The standard deviation of the chosen instances. 
+
     """
     _, std = regressor.predict(X, return_std=True, **predict_kwargs)
     std = std.reshape(X.shape[0], )
