@@ -6,17 +6,14 @@ Base classes for active learning algorithms
 import abc
 import sys
 import warnings
-from typing import Union, Callable, Optional, Tuple, List, Iterator, Any
+from typing import Any, Callable, Iterator, List, Tuple, Union
 
 import numpy as np
+import scipy.sparse as sp
+from modAL.utils.data import data_hstack, modALinput, retrieve_rows
 from sklearn.base import BaseEstimator
 from sklearn.ensemble._base import _BaseHeterogeneousEnsemble
 from sklearn.pipeline import Pipeline
-from sklearn.utils import check_X_y
-
-import scipy.sparse as sp
-
-from modAL.utils.data import data_vstack, data_hstack, modALinput, retrieve_rows
 
 if sys.version_info >= (3, 4):
     ABC = abc.ABC
@@ -32,12 +29,8 @@ class BaseLearner(ABC, BaseEstimator):
         estimator: The estimator to be used in the active learning loop.
         query_strategy: Function providing the query strategy for the active learning loop,
             for instance, modAL.uncertainty.uncertainty_sampling.
-        X_training: Initial training samples, if available.
-        y_training: Initial training labels corresponding to initial training samples.
         force_all_finite: When True, forces all values of the data finite.
             When False, accepts np.nan and np.inf values.
-        bootstrap_init: If initial training data is available, bootstrapping can be done during the first training.
-            Useful when building Committee models with bagging.
         on_transformed: Whether to transform samples with the pipeline defined by the estimator
             when applying the query strategy.
         **fit_kwargs: keyword arguments.
@@ -45,16 +38,11 @@ class BaseLearner(ABC, BaseEstimator):
     Attributes:
         estimator: The estimator to be used in the active learning loop.
         query_strategy: Function providing the query strategy for the active learning loop.
-        X_training: If the model hasn't been fitted yet it is None, otherwise it contains the samples
-            which the model has been trained on.
-        y_training: The labels corresponding to X_training.
     """
+
     def __init__(self,
                  estimator: BaseEstimator,
                  query_strategy: Callable,
-                 X_training: Optional[modALinput] = None,
-                 y_training: Optional[modALinput] = None,
-                 bootstrap_init: bool = False,
                  on_transformed: bool = False,
                  force_all_finite: bool = True,
                  **fit_kwargs
@@ -65,47 +53,10 @@ class BaseLearner(ABC, BaseEstimator):
         self.query_strategy = query_strategy
         self.on_transformed = on_transformed
 
-        self.X_training = X_training
-        self.Xt_training = None
-        self.y_training = y_training
-        if X_training is not None:
-            self._fit_to_known(bootstrap=bootstrap_init, **fit_kwargs)
-            self.Xt_training = self.transform_without_estimating(self.X_training) if self.on_transformed else None
-
-        assert isinstance(force_all_finite, bool), 'force_all_finite must be a bool'
+        assert isinstance(force_all_finite,
+                          bool), 'force_all_finite must be a bool'
         self.force_all_finite = force_all_finite
 
-    def _add_training_data(self, X: modALinput, y: modALinput) -> None:
-        """
-        Adds the new data and label to the known data, but does not retrain the model.
-
-        Args:
-            X: The new samples for which the labels are supplied by the expert.
-            y: Labels corresponding to the new instances in X.
-
-        Note:
-            If the classifier has been fitted, the features in X have to agree with the training samples which the
-            classifier has seen.
-        """
-        check_X_y(X, y, accept_sparse=True, ensure_2d=False, allow_nd=True, multi_output=True, dtype=None,
-                  force_all_finite=self.force_all_finite)
-
-        if self.X_training is None:
-            self.X_training = X
-            self.Xt_training = self.transform_without_estimating(self.X_training) if self.on_transformed else None
-            self.y_training = y
-        else:
-            try:
-                self.X_training = data_vstack((self.X_training, X))
-                self.Xt_training = data_vstack((
-                    self.Xt_training,
-                    self.transform_without_estimating(X)
-                )) if self.on_transformed else None
-                self.y_training = data_vstack((self.y_training, y))
-            except ValueError:
-                raise ValueError('the dimensions of the new training data and label must'
-                                 'agree with the training data and labels provided so far')
-
     def transform_without_estimating(self, X: modALinput) -> Union[np.ndarray, sp.csr_matrix]:
         """
         Transforms the data as supplied to the estimator.
@@ -135,7 +86,8 @@ class BaseLearner(ABC, BaseEstimator):
                 #       components but the final estimator, which is replaced by an empty (passthrough) component.
                 #       This prevents any special handling of the final transformation pipe, which is usually
                 #       expected to be an estimator.
-                transformation_pipe = pipe.__class__(steps=[*pipe.steps[:-1], ('passthrough', 'passthrough')])
+                transformation_pipe = pipe.__class__(
+                    steps=[*pipe.steps[:-1], ('passthrough', 'passthrough')])
                 Xt.append(transformation_pipe.transform(X))
 
         # in case no transformation pipelines are used by the estimator,
@@ -147,26 +99,6 @@ class BaseLearner(ABC, BaseEstimator):
         # concatenate all transformations and return
         return data_hstack(Xt)
 
-    def _fit_to_known(self, bootstrap: bool = False, **fit_kwargs) -> 'BaseLearner':
-        """
-        Fits self.estimator to the training data and labels provided to it so far.
-
-        Args:
-            bootstrap: If True, the method trains the model on a set bootstrapped from the known training instances.
-            **fit_kwargs: Keyword arguments to be passed to the fit method of the predictor.
-
-        Returns:
-            self
-        """
-        if not bootstrap:
-            self.estimator.fit(self.X_training, self.y_training, **fit_kwargs)
-        else:
-            n_instances = self.X_training.shape[0]
-            bootstrap_idx = np.random.choice(range(n_instances), n_instances, replace=True)
-            self.estimator.fit(self.X_training[bootstrap_idx], self.y_training[bootstrap_idx], **fit_kwargs)
-
-        return self
-
     def _fit_on_new(self, X: modALinput, y: modALinput, bootstrap: bool = False, **fit_kwargs) -> 'BaseLearner':
         """
         Fits self.estimator to the given data and labels.
@@ -180,41 +112,19 @@ class BaseLearner(ABC, BaseEstimator):
         Returns:
             self
         """
-        check_X_y(X, y, accept_sparse=True, ensure_2d=False, allow_nd=True, multi_output=True, dtype=None,
-                  force_all_finite=self.force_all_finite)
 
         if not bootstrap:
             self.estimator.fit(X, y, **fit_kwargs)
         else:
-            bootstrap_idx = np.random.choice(range(X.shape[0]), X.shape[0], replace=True)
+            bootstrap_idx = np.random.choice(
+                range(X.shape[0]), X.shape[0], replace=True)
             self.estimator.fit(X[bootstrap_idx], y[bootstrap_idx])
 
         return self
 
-    def fit(self, X: modALinput, y: modALinput, bootstrap: bool = False, **fit_kwargs) -> 'BaseLearner':
-        """
-        Interface for the fit method of the predictor. Fits the predictor to the supplied data, then stores it
-        internally for the active learning loop.
-
-        Args:
-            X: The samples to be fitted.
-            y: The corresponding labels.
-            bootstrap: If true, trains the estimator on a set bootstrapped from X.
-                Useful for building Committee models with bagging.
-            **fit_kwargs: Keyword arguments to be passed to the fit method of the predictor.
-
-        Note:
-            When using scikit-learn estimators, calling this method will make the ActiveLearner forget all training data
-            it has seen!
-
-        Returns:
-            self
-        """
-        check_X_y(X, y, accept_sparse=True, ensure_2d=False, allow_nd=True, multi_output=True, dtype=None,
-                  force_all_finite=self.force_all_finite)
-        self.X_training, self.y_training = X, y
-        self.Xt_training = self.transform_without_estimating(self.X_training) if self.on_transformed else None
-        return self._fit_to_known(bootstrap=bootstrap, **fit_kwargs)
+    @abc.abstractmethod
+    def fit(self, *args, **kwargs) -> None:
+        pass
 
     def predict(self, X: modALinput, **predict_kwargs) -> Any:
         """
@@ -242,12 +152,13 @@ class BaseLearner(ABC, BaseEstimator):
         """
         return self.estimator.predict_proba(X, **predict_proba_kwargs)
 
-    def query(self, X_pool, *query_args, **query_kwargs) -> Union[Tuple, modALinput]:
+    def query(self, X_pool, *query_args, return_metrics: bool = False, **query_kwargs) -> Union[Tuple, modALinput]:
         """
         Finds the n_instances most informative point in the data provided by calling the query_strategy function.
 
         Args:
             X_pool: Pool of unlabeled instances to retrieve most informative instances from
+            return_metrics: boolean to indicate, if the corresponding query metrics should be (not) returned
             *query_args: The arguments for the query strategy. For instance, in the case of
                 :func:`~modAL.uncertainty.uncertainty_sampling`, it is the pool of samples from which the query strategy
                 should choose instances to request labels.
@@ -257,16 +168,25 @@ class BaseLearner(ABC, BaseEstimator):
             Value of the query_strategy function. Should be the indices of the instances from the pool chosen to be
             labelled and the instances themselves. Can be different in other cases, for instance only the instance to be
             labelled upon query synthesis.
+            query_metrics: returns also the corresponding metrics, if return_metrics == True
         """
-        query_result = self.query_strategy(self, X_pool, *query_args, **query_kwargs)
 
-        if isinstance(query_result, tuple):
-            warnings.warn("Query strategies should no longer return the selected instances, "
-                          "this is now handled by the query method. "
-                          "Please return only the indices of the selected instances.", DeprecationWarning)
-            return query_result
+        try:
+            query_result, query_metrics = self.query_strategy(
+                self, X_pool, *query_args, **query_kwargs)
 
-        return query_result, retrieve_rows(X_pool, query_result)
+        except:
+            query_metrics = None
+            query_result = self.query_strategy(
+                self, X_pool, *query_args, **query_kwargs)
+
+        if return_metrics:
+            if query_metrics is None: 
+                warnings.warn(
+                "The selected query strategy doesn't support return_metrics")
+            return query_result, retrieve_rows(X_pool, query_result), query_metrics
+        else:
+            return query_result, retrieve_rows(X_pool, query_result)
 
     def score(self, X: modALinput, y: modALinput, **score_kwargs) -> Any:
         """
@@ -290,7 +210,6 @@ class BaseLearner(ABC, BaseEstimator):
 class BaseCommittee(ABC, BaseEstimator):
     """
     Base class for query-by-committee setup.
-
     Args:
         learner_list: List of ActiveLearner objects to form committee.
         query_strategy: Function to query labels.
@@ -316,11 +235,9 @@ class BaseCommittee(ABC, BaseEstimator):
     def _add_training_data(self, X: modALinput, y: modALinput) -> None:
         """
         Adds the new data and label to the known data for each learner, but does not retrain the model.
-
         Args:
             X: The new samples for which the labels are supplied by the expert.
             y: Labels corresponding to the new instances in X.
-
         Note:
             If the learners have been fitted, the features in X have to agree with the training samples which the
             classifier has seen.
@@ -331,7 +248,6 @@ class BaseCommittee(ABC, BaseEstimator):
     def _fit_to_known(self, bootstrap: bool = False, **fit_kwargs) -> None:
         """
         Fits all learners to the training data and labels provided to it so far.
-
         Args:
             bootstrap: If True, each estimator is trained on a bootstrapped dataset. Useful when
                 using bagging to build the ensemble.
@@ -343,7 +259,6 @@ class BaseCommittee(ABC, BaseEstimator):
     def _fit_on_new(self, X: modALinput, y: modALinput, bootstrap: bool = False, **fit_kwargs) -> None:
         """
         Fits all learners to the given data and labels.
-
         Args:
             X: The new samples for which the labels are supplied by the expert.
             y: Labels corresponding to the new instances in X.
@@ -358,9 +273,7 @@ class BaseCommittee(ABC, BaseEstimator):
         Fits every learner to a subset sampled with replacement from X. Calling this method makes the learner forget the
         data it has seen up until this point and replaces it with X! If you would like to perform bootstrapping on each
         learner using the data it has seen, use the method .rebag()!
-
         Calling this method makes the learner forget the data it has seen up until this point and replaces it with X!
-
         Args:
             X: The samples to be fitted on.
             y: The corresponding labels.
@@ -376,18 +289,18 @@ class BaseCommittee(ABC, BaseEstimator):
         Transforms the data as supplied to each learner's estimator and concatenates transformations.
         Args:
             X: dataset to be transformed
-
         Returns:
             Transformed data set
         """
         return data_hstack([learner.transform_without_estimating(X) for learner in self.learner_list])
 
-    def query(self, X_pool, *query_args, **query_kwargs) -> Union[Tuple, modALinput]:
+    def query(self, X_pool, return_metrics: bool = False, *query_args, **query_kwargs) -> Union[Tuple, modALinput]:
         """
         Finds the n_instances most informative point in the data provided by calling the query_strategy function.
 
         Args:
             X_pool: Pool of unlabeled instances to retrieve most informative instances from
+            return_metrics: boolean to indicate, if the corresponding query metrics should be (not) returned
             *query_args: The arguments for the query strategy. For instance, in the case of
                 :func:`~modAL.disagreement.max_disagreement_sampling`, it is the pool of samples from which the query.
                 strategy should choose instances to request labels.
@@ -397,25 +310,32 @@ class BaseCommittee(ABC, BaseEstimator):
             Return value of the query_strategy function. Should be the indices of the instances from the pool chosen to
             be labelled and the instances themselves. Can be different in other cases, for instance only the instance to
             be labelled upon query synthesis.
+            query_metrics: returns also the corresponding metrics, if return_metrics == True
         """
-        query_result = self.query_strategy(self, X_pool, *query_args, **query_kwargs)
 
-        if isinstance(query_result, tuple):
-            warnings.warn("Query strategies should no longer return the selected instances, "
-                          "this is now handled by the query method. "
-                          "Please return only the indices of the selected instances", DeprecationWarning)
-            return query_result
+        try:
+            query_result, query_metrics = self.query_strategy(
+                self, X_pool, *query_args, **query_kwargs)
 
-        return query_result, retrieve_rows(X_pool, query_result)
+        except:
+            query_metrics = None
+            query_result = self.query_strategy(
+                self, X_pool, *query_args, **query_kwargs)
+
+        if return_metrics:
+            if query_metrics is None: 
+                warnings.warn(
+                "The selected query strategy doesn't support return_metrics")
+            return query_result, retrieve_rows(X_pool, query_result), query_metrics
+        else:
+            return query_result, retrieve_rows(X_pool, query_result)
 
     def rebag(self, **fit_kwargs) -> None:
         """
         Refits every learner with a dataset bootstrapped from its training instances. Contrary to .bag(), it bootstraps
         the training data for each learner based on its own examples.
-
         Todo:
             Where is .bag()?
-
         Args:
             **fit_kwargs: Keyword arguments to be passed to the fit method of the predictor.
         """
@@ -424,7 +344,6 @@ class BaseCommittee(ABC, BaseEstimator):
     def teach(self, X: modALinput, y: modALinput, bootstrap: bool = False, only_new: bool = False, **fit_kwargs) -> None:
         """
         Adds X and y to the known training data for each learner and retrains learners with the augmented dataset.
-
         Args:
             X: The new samples for which the labels are supplied by the expert.
             y: Labels corresponding to the new instances in X.
@@ -444,4 +363,5 @@ class BaseCommittee(ABC, BaseEstimator):
 
     @abc.abstractmethod
     def vote(self, X: modALinput) -> Any:  # TODO: clarify typing
-        pass
\ No newline at end of file
+        pass
+
